@@ -1,3 +1,9 @@
+//! # Alpm
+//!
+//! See [`Alpm`] as the base type to interact with alpm.
+//!
+#![doc = include_str!("../README.md")]
+
 mod add;
 mod alpm;
 mod be_local;
